

// Include the SPI library
#include <SPI.h>

int RunOnce = 0; 

// Set output pins to control the digital pot and H-bridges
const int CS_PIN = 10;
const int A_ENABLE_PIN = 2; 
const int A_PHASE_PIN = 3; 

// Set up the variables to control voltage, level, frequency, period, and pulse
volatile float voltage;
volatile int level;
volatile int levelEstim; 
volatile int period_count; 
volatile float frequency;
volatile float period;
volatile float pulse;
volatile float pulseWidth;
volatile float time_low; 
volatile int num_per; 

// Define the command byte for use with the MCP42100 digital pot; see datasheet for additional details
const int commandByte = B11011111; 
int dataByte; 

// Definition of setVoltage function 
void setVoltage(float voltage)
{

    // Initializes SPI 
    SPI.begin();
  
    // Take the CS pin low to select the chip
    digitalWrite(CS_PIN, LOW);

   // Convert the voltage to 256 bit 
   dataByte = byte((voltage * 255 / 5)); 
    
   // Send in the command byte and the data byte via SPI
   SPI.transfer(commandByte);
   SPI.transfer(dataByte);

   // Take CS pin high to de-select the chip
   digitalWrite(CS_PIN, HIGH);

   // End SPI transaction 
   SPI.endTransaction(); 
  
}

// Definition of biphasicPulse function
void biphasicPulse(float pulseWidth, int PHASE_PIN, int ENABLE_PIN, float time_low) 
{
  digitalWrite(ENABLE_PIN, HIGH); 
  // PHASE_PIN is low by default so it does not need to be changed here
  delay((pulseWidth/2)) ;
  digitalWrite(ENABLE_PIN, LOW);
  digitalWrite(PHASE_PIN, HIGH); 
  digitalWrite(ENABLE_PIN, HIGH);
  delay((pulseWidth/2)) ;
  digitalWrite(ENABLE_PIN, LOW) ; // Stops stimulation
  digitalWrite(PHASE_PIN, LOW); // Returns PHASE to default

   // Leave off for remainder of period 
   delay(time_low);
 
}

// Definition of monophasicPulse function
void monophasicPulse(float pulseWidth, int PHASE_PIN, int ENABLE_PIN, float time_low) 
{
  digitalWrite(ENABLE_PIN, HIGH); 
  // PHASE_PIN is low by default so it does not need to be changed here
  delay((pulseWidth)) ;
  digitalWrite(ENABLE_PIN, LOW);
  digitalWrite(PHASE_PIN, LOW); // Returns PHASE to default

  // Leave off for remainder of period 
  delay(time_low);
 
}


// Main code

void setup() {

  Serial.begin(9600); 

  // Sets pins as outputs
  pinMode(CS_PIN, OUTPUT); 
  pinMode(A_ENABLE_PIN, OUTPUT);
  pinMode(A_PHASE_PIN, OUTPUT); 

  // Ensure stimulation is off at beginning 
  digitalWrite(A_ENABLE_PIN, LOW); 

  // Set default voltage to 0
  voltage = 0; 
  setVoltage(voltage); 

  // Set PHASE_PIN default to low
  digitalWrite(A_PHASE_PIN, LOW); 
}

/* Main control loop */

void loop()
{

  int serialIn = 0;

  // Read strings from serial
  if (Serial.available() > 0)
  {
    serialIn = Serial.read(); 
  }

  // ****************************************************************************

  // If combined ET/MCR/FFR is read 
  if (serialIn == 65) // 65 is "A" in ASCII
  {
    //5 Seconds without stimulation 

  delay(5000) ; 

  // Initial Settings
  voltage = 5;
  pulseWidth = 5; 
  frequency = 1; 
  period = 1000 / frequency;
  time_low = period - pulseWidth; 
 
  // Cycle through voltages 
  for( int cycle_count = 0; cycle_count < 10; cycle_count++) 
  {
    voltage = 5 - (cycle_count * 0.5); 
    setVoltage(voltage);

    // Hold at voltage for 5 s 
    for (int counter = 0; counter < 5; counter++)
    {
      monophasicPulse(pulseWidth, A_PHASE_PIN, A_ENABLE_PIN, time_low) ; 
    }
  }

    // Settings for frequency sweep
  voltage = 5;
  setVoltage(voltage); 
  pulseWidth = 5; 
  frequency = 1; 
  period = 1000 / frequency;
  time_low = period - pulseWidth;

  // Cycle through frequencies 0.5 to 4 Hz by 0.5 Hz
  for( float cycle_count_1 = 0; cycle_count_1 < 8; cycle_count_1++)
  { 
    frequency = ((1 + (cycle_count_1)) / 2.0); 
    period = 1000 / frequency; 
    time_low = period - pulseWidth; 

    num_per = 20*frequency; 

    // Holds for 20 seconds
    for (int counter = 0; counter < num_per; counter++)
    {
      monophasicPulse(pulseWidth, A_PHASE_PIN, A_ENABLE_PIN, time_low) ;
    }

  }

  // Post Rest Potentiation 

  delay(20000); 

  // PRP (5 seconds) 
  frequency = 1; 
  period = 1000 / frequency; 
  time_low = period - pulseWidth; 
  for (int counter = 0; counter < 5; counter++)
  {
    monophasicPulse(pulseWidth, A_PHASE_PIN, A_ENABLE_PIN, time_low) ; 
  }
 
  // Turn off voltage at end 
  voltage = 0; 
  setVoltage(voltage); 
  
  }

  // ****************************************************************************
  
}
