  
/*
  June 11, 2021
  5 H-bridges and 1 digital potentiameter are used to control 4 biphasic output channels 
  All 4 channels have the same voltage but can differ in frequency 
  Modifierd from previous version to ensure that pulses are not concurrent

  Channels A and B will ramp by 0.33 Hz every day, while channels C and D will remain at 2 Hz. 

  On the physical Arduino shield, A and B are on the left and C and D are on the right when
  oriented such that the stimulator output wires are pointing forward. 
*/ 



// Include the SPI library
#include <SPI.h>

// Set output pins to control the digital pot and H-bridges
const int CS_PIN = 10;
const int B_ENABLE_PIN = 0; 
const int B_PHASE_PIN = 1; 
const int A_ENABLE_PIN = 2; 
const int A_PHASE_PIN = 3; 
const int D_ENABLE_PIN = 4;
const int D_PHASE_PIN = 5; 
const int C_ENABLE_PIN = 6;
const int C_PHASE_PIN = 7; 



// Define the command byte for use with the MCP42100 digital pot; see datasheet for additional details
const int commandByte = B11011111; 
int dataByte; 

// Define stimulation parameters 
float stimVoltage = 2.5; //in V

const float stimStart = 2; // in Hz

float stimFrequencyA = 2;
float stimFrequencyB = 2; 
float stimFrequencyC = 2;
float stimFrequencyD = 2;

float stimPulseWidth = 2; //in milliseconds
// Note: stimPulseWidth is the total pulse width for the combined positive and negative phases

float time_off_A = ((1/stimFrequencyA)*1000) - stimPulseWidth; 
float time_off_B = ((1/stimFrequencyB)*1000) - stimPulseWidth; 
float time_off_C = ((1/stimFrequencyC)*1000) - stimPulseWidth; 
float time_off_D = ((1/stimFrequencyD)*1000) - stimPulseWidth; 

// Initialize variables for tracking the time 
unsigned long current_time; 
float day_counter;
unsigned long day_start; 
unsigned long pulse_off_A = 0; 
unsigned long pulse_off_B = 0; 
unsigned long pulse_off_C = 0; 
unsigned long pulse_off_D = 0; 

// Definition of setVoltage function 
void setVoltage(float voltage)
{

    // Initializes SPI 
    SPI.begin();
  
    // Take the CS pin low to select the chip
    digitalWrite(CS_PIN, LOW);

   // Convert the voltage to 256 bit 
   dataByte = byte((voltage * 255 / 5)); 
    
   // Send in the command byte and the data byte via SPI
   SPI.transfer(commandByte);
   SPI.transfer(dataByte);

   // Take CS pin high to de-select the chip
   digitalWrite(CS_PIN, HIGH);

   // End SPI transaction 
   SPI.endTransaction(); 
  
}

// Definition of biphasicPulse function
void biphasicPulse(float pulseWidth, int PHASE_PIN, int ENABLE_PIN) 
{
  digitalWrite(ENABLE_PIN, HIGH); 
  // PHASE_PIN is low by default so it does not need to be changed here
  delay((pulseWidth/2)) ;
  digitalWrite(ENABLE_PIN, LOW);
  // delay((pulseWidth/2)); // Creates a separation between phases of pulse 
  digitalWrite(PHASE_PIN, HIGH); 
  digitalWrite(ENABLE_PIN, HIGH);
  delay((pulseWidth/2)) ;
  digitalWrite(ENABLE_PIN, LOW) ; // Stops stimulation
  digitalWrite(PHASE_PIN, LOW); // Returns PHASE to default
  // delay((pulseWidth/2));
 
}


// Main code

void setup() {

  // Sets pins as outputs
  pinMode(CS_PIN, OUTPUT); 
  pinMode(A_ENABLE_PIN, OUTPUT);
  pinMode(A_PHASE_PIN, OUTPUT); 
  pinMode(B_ENABLE_PIN, OUTPUT); 
  pinMode(B_PHASE_PIN, OUTPUT); 
  pinMode(C_ENABLE_PIN, OUTPUT);
  pinMode(C_PHASE_PIN, OUTPUT); 
  pinMode(D_ENABLE_PIN, OUTPUT); 
  pinMode(D_PHASE_PIN, OUTPUT); 
  
  // Ensure stimulation is off at beginning 
  digitalWrite(A_ENABLE_PIN, LOW); 
  digitalWrite(B_ENABLE_PIN, LOW); 
  digitalWrite(C_ENABLE_PIN, LOW); 
  digitalWrite(D_ENABLE_PIN, LOW); 

  // Set PHASE_PIN default to low
  digitalWrite(A_PHASE_PIN, LOW); 
  digitalWrite(B_PHASE_PIN, LOW); 
  digitalWrite(C_PHASE_PIN, LOW); 
  digitalWrite(D_PHASE_PIN, LOW); 
  
  // Define the stimulation voltage
  setVoltage(stimVoltage); 

  // Start tracking the days since the program started 
  day_counter = 0.0; 
  day_start = 0; 

}

void loop() {

  // Check if A needs to pulse 
  current_time = millis(); 
  if ((current_time - pulse_off_A) > time_off_A){
    biphasicPulse(stimPulseWidth, A_PHASE_PIN, A_ENABLE_PIN); 
    pulse_off_A = millis(); 
  }
  
  // Check if B needs to pulse
  current_time = millis(); 
  if ((current_time - pulse_off_B) > time_off_B){
    biphasicPulse(stimPulseWidth, B_PHASE_PIN, B_ENABLE_PIN); 
    pulse_off_B = millis(); 
  }

   // Check if C needs to pulse
  current_time = millis(); 
  if ((current_time - pulse_off_C) > time_off_C){
    biphasicPulse(stimPulseWidth, C_PHASE_PIN, C_ENABLE_PIN); 
    pulse_off_C = millis(); 
  }

   // Check if D needs to pulse
  current_time = millis(); 
  if ((current_time - pulse_off_D) > time_off_D){
    biphasicPulse(stimPulseWidth, D_PHASE_PIN, D_ENABLE_PIN); 
    pulse_off_D = millis(); 
  }

  // Check for new day 
  current_time = millis(); 
  if (current_time - day_start > 86400000){ 
    day_counter = day_counter + 1; 
    
    stimFrequencyA = stimStart + (day_counter/3.0); 
    time_off_A = ((1.0/stimFrequencyA)*1000) - stimPulseWidth; 
    
    stimFrequencyC = 2.0; 
    time_off_B = ((1.0/stimFrequencyB)*1000) - stimPulseWidth; 
    
    stimFrequencyB = stimStart + (day_counter/3.0); 
    time_off_C = ((1.0/stimFrequencyC)*1000) - stimPulseWidth; 
    
    stimFrequencyD = 2.0; 
    time_off_D = ((1.0/stimFrequencyD)*1000) - stimPulseWidth; 
    
    day_start = millis(); 
  }

  
}
